/*
*  Copyright (C) 2000 Brian Harris
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2, or (at your option)
*  any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

// this handles plugin loading/unloading

#include "StdAfx.h"

#include "MfcEmbed.h"
extern CMfcEmbedApp theApp;

#include "BrowserView.h"
#include "BrowserFrm.h"

#include "kmeleon_plugin.h"
#include "Plugins.h"
#include "Utils.h"

int SessionSize=0;
char **pHistory;
char **pHistUrl;
kmeleonDocInfo kDocInfo;
kmeleonPointInfo gPointInfo;


static char *safe_strdup(const char *ptr) {
  if (ptr)
    return strdup(ptr);
  return NULL;
}

CPlugins::CPlugins()
{
}

CPlugins::~CPlugins()
{

   delete gPointInfo.image;
   delete gPointInfo.link;
   delete gPointInfo.frame;
   delete gPointInfo.page;

   if (SessionSize) {
     for (int i=0; i<SessionSize; i++) {
       if (pHistory && pHistory[i])
         delete pHistory[i];
       if (pHistUrl && pHistUrl[i])
          delete pHistUrl[i];
     }
   }
   if (pHistory)
      delete pHistory;
   if (pHistUrl)
      delete pHistUrl;
   pHistory = NULL;
   pHistUrl = NULL;

   if (kDocInfo.url)
      delete kDocInfo.url;
   
   if (kDocInfo.title)
      delete kDocInfo.title;
   
   UnLoadAll();
}

// returns a pointer to the char after the last \ or /
const char *FileNoPath(const char *filepath)
{
   char *p1 = strrchr(filepath, '\\');
   char *p2 = strrchr(filepath, '/');
   if (p1 > p2) {
      return p1 + 1;
   }
   else if (p2 > p1) {
      return p2 + 1;
   }
   else {
      return filepath;
   }
}


UINT currentCmdID = PLUGIN_COMMAND_START_ID;
UINT GetCommandIDs(int num)
{
   UINT freeID = currentCmdID;
   currentCmdID += num;
   return freeID;
}

int CPlugins::OnUpdate(UINT command)
{
   if (command >= PLUGIN_COMMAND_START_ID && command <= currentCmdID)
      return true;
   return false;
}

HWND NavigateTo(const char *url, int windowState, HWND mainWnd)
{
   CBrowserFrame *mainFrame, *newFrame = NULL;
   if (mainWnd)
      mainFrame = (CBrowserFrame *)CWnd::FromHandle(mainWnd);
   else
      mainFrame = theApp.m_pMostRecentBrowserFrame;

   if (!mainFrame) {
      return NULL;
   }
   nsEmbedString str;

   switch(windowState&15) {
   case OPEN_NORMAL:
      mainFrame->m_wndBrowserView.OpenURL(url);
      break;
   case OPEN_NEW:
      NS_CStringToUTF16(nsEmbedCString(url), NS_CSTRING_ENCODING_ASCII, str);
      newFrame = mainFrame->m_wndBrowserView.OpenURLInNewWindow(str.get());
      break;
   case OPEN_BACKGROUND:
	  NS_CStringToUTF16(nsEmbedCString(url), NS_CSTRING_ENCODING_ASCII, str);
      newFrame = mainFrame->m_wndBrowserView.OpenURLInNewWindow(str.get(), true);
      break;
   }
   if (newFrame && (windowState & OPEN_CLONE))
	   mainFrame->m_wndBrowserView.CloneSHistory(newFrame->m_wndBrowserView);

   return newFrame ? newFrame->m_hWnd:NULL;
}

void _NavigateTo(const char *url, int windowState, HWND mainWnd)
{
	NavigateTo(url, windowState, mainWnd);
}

kmeleonDocInfo * GetDocInfo(HWND mainWnd)
{
   CBrowserFrame *frame;
   if (mainWnd)
	   frame = (CBrowserFrame *)CWnd::FromHandle(mainWnd);
   else
	   frame = theApp.m_pMostRecentBrowserFrame;

   if (!frame)
      return NULL;

   USES_CONVERSION;

   CString url;
   frame->m_wndBrowserView.GetCurrentURI(url);
   char* docurl = new char[url.GetLength()+1];
   strcpy(docurl, T2CA(url));

   CString title;
   frame->m_wndBrowserView.GetPageTitle(title);
   char* doctitle = new char[title.GetLength()+1];
   strcpy(doctitle, T2CA(title));

   if (kDocInfo.url)
      delete kDocInfo.url;
   
   if (kDocInfo.title)
      delete kDocInfo.title;

   kDocInfo.title = doctitle;
   kDocInfo.url = docurl;
#ifdef INTERNAL_SITEICONS
   kDocInfo.idxIcon = theApp.favicons.GetIcon(frame->m_wndBrowserView.m_IconUri);
#endif

   return &kDocInfo;
}

HIMAGELIST GetIconList()
{
#ifdef INTERNAL_SITEICONS
	return theApp.favicons.GetSafeHandle();
#else
	return NULL;
#endif
}

long GetPreference(enum PREFTYPE type, const char *preference, void *ret, void *defVal)
{
   long result;
   switch (type) {
   case PREF_BOOL:
      *(int *)ret = result = theApp.preferences.GetBool(preference, *(int *)defVal);
      break;
   case PREF_INT:
      *(int *)ret = result = theApp.preferences.GetInt(preference, *(int *)defVal);
      break;
   case PREF_STRING:
      result = theApp.preferences.GetString(preference, (char *)ret, (char *)defVal);
      break;
   case PREF_UNISTRING:
      result = theApp.preferences.GetString(preference, (wchar_t *)ret, (wchar_t *)defVal);
      break;
   }
   return result;
}

void _GetPreference(enum PREFTYPE type, char *preference, void *ret, void *defVal)
{
   GetPreference(type, preference, ret, defVal);
}

void SetPreference(enum PREFTYPE type, const char *preference, void *val, BOOL update)
{
   theApp.preferences.Save(false);

   switch (type) {
      case PREF_BOOL:
         theApp.preferences.SetBool(preference, *(int *)val);
         break;
      case PREF_INT:
         theApp.preferences.SetInt(preference, *(int *)val);
         break;
      case PREF_STRING:
         theApp.preferences.SetString(preference, (char *)val);
         break;
      case PREF_UNISTRING:
         theApp.preferences.SetString(preference, (wchar_t *)val);
         break;
   }

   theApp.preferences.Flush();
   theApp.preferences.Load();
}

void DelPreference(char *preference)
{
   theApp.preferences.Clear(preference);
   theApp.preferences.Flush();
   theApp.preferences.Load();
}

void SetStatusBarText(const char *s) {
   USES_CONVERSION;
   if (theApp.m_pMostRecentBrowserFrame) // Yes, it can happen
     theApp.m_pMostRecentBrowserFrame->m_wndStatusBar.SetPaneText(0, A2CT(s));
}

int GetMozillaSessionHistory (char ***titles, char ***urls, int *count, int *index)
{
   nsresult result;
   int i;

   if (!theApp.m_pMostRecentBrowserFrame || !theApp.m_pMostRecentBrowserFrame->m_wndBrowserView.mWebNav) 
	   return FALSE;
   
   nsCOMPtr<nsISHistory> h;

   result = theApp.m_pMostRecentBrowserFrame->m_wndBrowserView.mWebNav->GetSessionHistory(getter_AddRefs (h));

   if (!NS_SUCCEEDED (result) || (!h)) return FALSE;
   
   h->GetCount (count);
   h->GetIndex (index);

   // Clear the previous table
   if (SessionSize) {
      for (i=0; i<SessionSize; i++) {
         if (pHistory && pHistory[i])
            delete pHistory[i];
         if (pHistUrl && pHistUrl[i])
            delete pHistUrl[i];
      }
   }
   
   SessionSize = *count;
   
   if (pHistory)
      delete pHistory;
   pHistory = new char *[SessionSize];
   
   nsCOMPtr<nsIHistoryEntry> he;
   PRUnichar *title;
   
   if (pHistUrl)
      delete pHistUrl;
   pHistUrl = new char *[SessionSize];
   
   nsCOMPtr<nsIURI> theUri;
   nsCString uri;
   
   for (i=0; i < SessionSize; i++) {
      pHistory[i] = NULL;
      pHistUrl[i] = NULL;
   }
   
   for (i=0; i < SessionSize; i++) {
      result = h->GetEntryAtIndex(i, PR_FALSE, getter_AddRefs (he));
      if (!NS_SUCCEEDED(result) || (!he)) return FALSE;
      
      result = he->GetURI(getter_AddRefs(theUri));
      if (!NS_SUCCEEDED(result) || (!theUri)) return FALSE;
      
      theUri->GetSpec(uri);
      char *t = strdup(uri.get());

      pHistUrl[i] = t;
      
      result = he->GetTitle (&title);
      if (!NS_SUCCEEDED(result) || (!title)) return FALSE;
      
      // The title is in 16-bit unicode, this converts it to 8bit (UTF)
      int len;
      len = WideCharToMultiByte(CP_ACP, 0, title, -1, 0, 0, NULL, NULL);
      char *s = new char[len+1];
      len = WideCharToMultiByte(CP_ACP, 0, title, -1, s, len, NULL, NULL);
      s[len] = 0;
      pHistory[i] = s;
	  nsMemory::Free(title);
   }
   
   if (titles)
      *titles = pHistory;
   if (urls)
      *urls = pHistUrl;

   return TRUE;
}

void GotoHistoryIndex(UINT index)
{
	CBrowserFrame	*mainFrame = theApp.m_pMostRecentBrowserFrame;
	if (mainFrame)
		mainFrame->m_wndBrowserView.mWebNav->GotoIndex(index);
}

void RegisterBand(HWND hWnd, char *name, int visibleOnMenu)
{
   USES_CONVERSION;
   theApp.m_pMostRecentBrowserFrame->m_wndReBar.RegisterBand(hWnd, A2T(name), visibleOnMenu);
}

// This lets a plugin create a toolbar within the current browser frame
// the advantage of having K-Meleon create the toolbar is that it will
// be handled through MFC, which will handle the button states through
// UPDATE_UI calls
HWND CreateToolbar(HWND hWnd, UINT style) {
   CBrowserFrame *browserFrm = (CBrowserFrame *)CWnd::FromHandle(hWnd);

   if (browserFrm)
      return browserFrm->CreateToolbar(style);
   else
      return NULL;
}

int GetID(char *strID) {

   int ID = theApp.GetID(strID);

   if (!ID) {
      char *plugin = strID;
      char *parameter = strchr(strID, '(');
      if (parameter) {
         *parameter = 0;

		 int c = 1;
		 char* close = parameter++;

		 while ( *(++close) ) {
			 if (*close == '(')
				 c++;
			 else if (*close == ')') {
				 c--;
				 if (!c) break;
			 }
		 }
		 *close = 0;
      }
      
      theApp.plugins.SendMessage(plugin, "* GetID", "DoAccel", (long) parameter, (long)&ID);
   }

   return ID;
}

kmeleonPointInfo *GetInfoAtNode(nsIDOMNode* aNode)
{
   delete gPointInfo.image;
   delete gPointInfo.link;
   delete gPointInfo.frame;
   delete gPointInfo.page;
   gPointInfo.image = NULL;
   gPointInfo.link  = NULL;
   gPointInfo.frame = NULL;
   gPointInfo.page  = NULL;
	
   if (!aNode) 
		return &gPointInfo;

   if (!theApp.m_pMostRecentBrowserFrame || !theApp.m_pMostRecentBrowserFrame->m_wndBrowserView)
      return &gPointInfo;
      
   CBrowserView *pBrowserView;  
   pBrowserView = &theApp.m_pMostRecentBrowserFrame->m_wndBrowserView;

   if (!pBrowserView)
      return &gPointInfo;

   // get the page url
   USES_CONVERSION;
   CString url;
   pBrowserView->GetCurrentURI(url);
   gPointInfo.page = strdup(T2CA(url));

   nsEmbedString strBuf;
   nsresult rv = NS_OK;


   // check if there's a link
   // Search for an anchor element
   nsCOMPtr<nsIDOMHTMLAnchorElement> linkElement;
   nsCOMPtr<nsIDOMNode> node = aNode;
   while (node) {
      linkElement = do_QueryInterface(node);
      if (linkElement)
         break;
      
      nsCOMPtr<nsIDOMNode> parentNode;
      node->GetParentNode(getter_AddRefs(parentNode));
      node = parentNode;
   }
   if (linkElement) {
      rv = linkElement->GetHref(strBuf);
      if(NS_SUCCEEDED(rv)) {
         if (strBuf.Length()) {
            gPointInfo.link = new char[strBuf.Length() + 1];
			UTF16ToCString(strBuf, gPointInfo.link);
         }
      }
   }


   // check for an image
   nsCOMPtr<nsIDOMHTMLImageElement> imgPointInfo(do_QueryInterface(aNode, &rv));
   if(NS_SUCCEEDED(rv)) {
      rv = imgPointInfo->GetSrc(strBuf);
      if(NS_SUCCEEDED(rv)) {
         gPointInfo.image = new char[strBuf.Length() + 1];
		 UTF16ToCString(strBuf, gPointInfo.image);
      }
   }


   // get the current Frame URL
   nsCOMPtr<nsIDOMDocument> domDoc;
   rv = aNode->GetOwnerDocument(getter_AddRefs(domDoc));
   
   if(NS_SUCCEEDED(rv)) {
      nsCOMPtr<nsIDOMHTMLDocument> htmlDoc(do_QueryInterface(domDoc, &rv));
      if(NS_SUCCEEDED(rv)) {
         rv = htmlDoc->GetURL(strBuf);
         if(NS_SUCCEEDED(rv)) {
            gPointInfo.frame = new char[strBuf.Length() +1];
			UTF16ToCString(strBuf, gPointInfo.frame);
         }
      }
   }


   return &gPointInfo;
}

kmeleonPointInfo *GetInfoAtClick(HWND hWnd)
{
	CBrowserFrame *frame;
	frame = (CBrowserFrame *)CWnd::FromHandle(hWnd);

	if (!frame || !::IsWindow(frame->m_wndBrowserView.m_hWnd)) {
      return GetInfoAtNode(nsnull);
   }

   if (!theApp.m_pMostRecentBrowserFrame || !theApp.m_pMostRecentBrowserFrame->m_wndBrowserView)
	   return GetInfoAtNode(nsnull);

   return GetInfoAtNode(frame->m_wndBrowserView.m_lastMouseActionNode);
}

kmeleonPointInfo *GetInfoAtPoint(int x, int y)
{

   if (!theApp.m_pMostRecentBrowserFrame || !theApp.m_pMostRecentBrowserFrame->m_wndBrowserView)
	   return GetInfoAtNode(nsnull);

   CBrowserView *pBrowserView;  
   pBrowserView = &theApp.m_pMostRecentBrowserFrame->m_wndBrowserView;

   // get the DOMNode at the point
   nsCOMPtr<nsIDOMNode> aNode;
   aNode = pBrowserView->GetNodeAtPoint(x, y, TRUE);
   return GetInfoAtNode(aNode);
}

// return 0 if the function did not succeed (ie, trying to open a link from
// a point that is not a link)
// return 1 if it does succeed

int CommandAtPoint(int command, WORD x, WORD y) {
   CBrowserView *pBrowserView;  
   pBrowserView = &theApp.m_pMostRecentBrowserFrame->m_wndBrowserView;


   if (!pBrowserView)
      return FALSE;

   nsCOMPtr<nsIWebBrowserFocus> focus(do_GetInterface(pBrowserView->mWebBrowser));
   if(!focus)
      return FALSE;

   focus->Activate();

   
   pBrowserView->GetNodeAtPoint(x, y, TRUE);

   switch (command) {
   case ID_OPEN_LINK:
   case ID_OPEN_LINK_IN_NEW_WINDOW:
   case ID_OPEN_LINK_IN_BACKGROUND:
   case ID_SAVE_LINK_AS:
   case ID_COPY_LINK_LOCATION:
      if (!pBrowserView->mCtxMenuLinkUrl.Length())
         return 0;
      break;

   case ID_VIEW_IMAGE:
   case ID_SAVE_IMAGE_AS:
   case ID_COPY_IMAGE_LOCATION:
   case ID_COPY_IMAGE_CONTENT:
      if (!pBrowserView->mCtxMenuImgSrc.Length())
         return 0;
      break;

   case ID_OPEN_FRAME:
   case ID_OPEN_FRAME_IN_BACKGROUND:
   case ID_OPEN_FRAME_IN_NEW_WINDOW:
   case ID_VIEW_FRAME_SOURCE:
      if (!pBrowserView->mCtxMenuCurrentFrameURL.Length())
         return 0;
      break;
   }

   pBrowserView->mpBrowserFrame->PostMessage(WM_COMMAND, command, NULL);
   return 1;
}

int SetGlobalVar(enum PREFTYPE type, const char *preference, void *value)
{
	if (!theApp.m_pMostRecentBrowserFrame || !value || !preference) 
		return 0;

   int ret = 0;

   CBrowserView *pBrowserView;  
   pBrowserView = &theApp.m_pMostRecentBrowserFrame->m_wndBrowserView;

   switch (type) {
   case PREF_STRING:
	   if (!stricmp(preference, "URLBAR")) {
		   theApp.m_pMostRecentBrowserFrame->m_wndUrlBar.EditChanged(FALSE);
         USES_CONVERSION;
		   theApp.m_pMostRecentBrowserFrame->m_wndUrlBar.SetCurrentURL(A2CT((char*)value));
		   ret = 1;
	   }
	   else if (!stricmp(preference, "CHARSET")) {
         USES_CONVERSION;
         pBrowserView->ForceCharset((char*)value);
		   ret = 1;
      }
	   else if (!stricmp(preference, "TITLE")) {
		   if (pBrowserView->mpBrowserFrameGlue) {
		      USES_CONVERSION;
		      pBrowserView->mpBrowserFrameGlue->SetBrowserFrameTitle(A2CW((char*)value));
		   }
		   ret = 1;
      }
	   break;
   case PREF_INT:
	   if (!stricmp(preference, "TextZoom")) {
		 int zoom = pBrowserView->GetTextSize();
		 pBrowserView->ChangeTextSize(*(int*)value - zoom);
		 ret = 1;
	   }
   }
   return ret;
}

int GetGlobalVar(enum PREFTYPE type, char *preference, void *ret) {

   if (!theApp.m_pMostRecentBrowserFrame) 
     return -1;

   int retLen = -1;

   CBrowserView *pBrowserView;  
   pBrowserView = &theApp.m_pMostRecentBrowserFrame->m_wndBrowserView;

   switch (type) {
   case PREF_UNISTRING:
	  if (!stricmp(preference, "SelectedText")) {
		nsEmbedString sel;  
		pBrowserView->GetUSelection(sel);
		retLen = sel.Length();
		 if (ret)
			 wcscpy((wchar_t*)ret, sel.get());
	  }
	  break;
   case PREF_STRING:
	  if (!stricmp(preference, "URL")) {
	     CString url;
		 theApp.m_pMostRecentBrowserFrame->m_wndBrowserView.GetCurrentURI(url);
		 retLen = url.GetLength();
		 if (ret) {
			 USES_CONVERSION;
			 strcpy((char*)ret, T2CA(url));
		 }
      }
	  else if (!stricmp(preference, "URLBAR")) {
		 CString url;
		 theApp.m_pMostRecentBrowserFrame->m_wndUrlBar.GetEnteredURL(url);
         retLen = url.GetLength();
		 if (ret) {
			 USES_CONVERSION;
			 strcpy((char*)ret, T2CA(url));
		 }
      }
      else if (!stricmp(preference, "LinkURL")) {
         retLen = pBrowserView->mCtxMenuLinkUrl.Length();
         if (ret) 
			 UTF16ToCString(pBrowserView->mCtxMenuLinkUrl, (char*)ret);
      }
      else if (!stricmp(preference, "ImageURL")) {
         retLen = pBrowserView->mCtxMenuImgSrc.Length();
         if (ret) 
			 UTF16ToCString(pBrowserView->mCtxMenuImgSrc, (char*)ret);
      }
      else if (!stricmp(preference, "FrameURL")) {
         retLen = pBrowserView->mCtxMenuCurrentFrameURL.Length();
         if (ret)
			 UTF16ToCString(pBrowserView->mCtxMenuCurrentFrameURL, (char*)ret);
      }
      else if (!stricmp(preference, "TITLE")) {
         CString title;
         pBrowserView->GetPageTitle(title);
		 USES_CONVERSION;
         retLen = title.GetLength();
         if (ret) strcpy((char *)ret, T2CA(title));
      }
	  else if (!stricmp(preference, "SelectedText")) {
		CString sel;  
		pBrowserView->GetSelection(sel);
		retLen = sel.GetLength();
		 if (ret) {
			 USES_CONVERSION;
			 strcpy((char*)ret, T2CA(sel));
		 }
	  }
	  else if (!stricmp(preference, "CHARSET")) {
		 char charset[64] = {0};
         pBrowserView->GetCharset(charset);
		 USES_CONVERSION;
         retLen = strlen(charset);
		 if (ret) {
			 strcpy((char *)ret, charset);
			 for(int i=0;i<retLen;i++)
				((char*)ret)[i] = tolower(((char*)ret)[i]);
		 }
      }
	  break;

	case PREF_INT:
	  if (!stricmp(preference, "TextZoom"))  {
		  int tz = pBrowserView->GetTextSize();
		  if (ret) *(int*)ret = tz;
		  return 0;
	  }
      break;
   }
    
   return retLen;
}

char *EncodeUTF8(const char *str) {
  USES_CONVERSION;
  nsEmbedString aStr;
  aStr.Append(A2W(str));
  nsEmbedCString _str;
  NS_UTF16ToCString(aStr, NS_CSTRING_ENCODING_UTF8, _str);
  char *pszStr = safe_strdup(_str.get());
  return pszStr;
}

char *DecodeUTF8(const char *str) {
  USES_CONVERSION;
  nsEmbedString _str;
  NS_CStringToUTF16(nsEmbedCString(str), NS_CSTRING_ENCODING_UTF8, _str);
  char *pszStr = safe_strdup(W2CA(_str.get()));
  return pszStr;
}

void GetBrowserviewRect(HWND mainWnd, RECT *rc) {
   CBrowserFrame *frame = (CBrowserFrame *)CWnd::FromHandle(mainWnd);
   if (frame)
      frame->m_wndBrowserView.GetWindowRect(rc);
}

HMENU GetMenu(char *menuName){
   CMenu *menu;
   USES_CONVERSION;
   menu = theApp.menus.GetMenu(A2T(menuName));

   return menu ? menu->m_hMenu : NULL;
}

void SetForceCharset(char *aCharset) {
   if (theApp.m_pMostRecentBrowserFrame)
     theApp.m_pMostRecentBrowserFrame->m_wndBrowserView.ForceCharset(aCharset);
}

void SetCheck(int id, BOOL mark) {
  theApp.menus.SetCheck(id, mark);
}

void ClearCache(int cache) {
   nsresult rv;

   nsCOMPtr<nsICacheService> CacheService =
      do_GetService(NS_CACHESERVICE_CONTRACTID, &rv);
   if (NS_FAILED(rv)) return;

   CacheService->EvictEntries(cache);
}

void BroadcastMessage(UINT Msg, WPARAM wParam, LPARAM lParam) {
  theApp.BroadcastMessage(Msg, wParam, lParam);
}

void ParseAccel(char *str) {
  theApp.accel.Parse(str);
}

kmeleonPlugin * Load(char *kplugin) {
  USES_CONVERSION;
  return theApp.plugins.Load(CString(A2T(kplugin)));
}

long CPlugins::SendMessage(const char *to, const char *from, const char *subject, long data1, long data2)
{
   long retVal = 0;

   POSITION pos = pluginList.GetStartPosition();
   kmeleonPlugin * kPlugin;
   CString s;
   while (pos) {
      pluginList.GetNextAssoc( pos, s, kPlugin);
      if (kPlugin->loaded && kPlugin->DoMessage) {
         retVal += kPlugin->DoMessage(to, from, subject, data1, data2);
      }
   }
   return retVal;
}

long SendMessage(const char *to, const char *from, const char *subject, long data1, long data2)
{
   // * is reserved for internal k-meleon messages.  plugins may not use it
   if (from[0] == '*') {
      return 0;
   }

   return theApp.plugins.SendMessage(to, from, subject, data1, data2);
}

int RegisterSideBar (HWND mainWnd, TCHAR *name, SideBarInitProc proc, int commandID, int visibleOnMenu)
{
#ifdef INTERNAL_SIDEBAR
	CSideBar *sidebar = (CSideBar *)CWnd::FromHandle(mainWnd);
	USES_CONVERSION;
	return sidebar->RegisterSideBar(name, proc, commandID, visibleOnMenu);
#else
	return -1;
#endif
}

void ToggleSideBar (HWND mainWnd, int index)
{
#ifdef INTERNAL_SIDEBAR
	CBrowserFrame *frame = (CBrowserFrame *)CWnd::FromHandle(mainWnd);
	frame->m_wndSideBar.ToggleVisibility(index);
#endif
}

const char* Translate(const char* text)
{
	USES_CONVERSION;
#ifndef _UNICODE
	return theApp.lang.Translate(A2CT(text));
#else
   static char* Translated = NULL;
   if (Translated) {
      free(Translated);
      Translated = NULL;
   }

   const WCHAR* uText = A2CW(text);
   const WCHAR* uTranslated = theApp.lang.Translate(uText);
   if (uTranslated == uText)
      return text;

   Translated = strdup(W2CA(uTranslated));
   return Translated;
#endif
}

int TranslateEx(const char* originalText,  TCHAR* translatedText, int bufferlen, BOOL forMenu)
{
	CString csTrans;
	char* accel = 0;
	int IsTranslated;

	if (forMenu)
	{
	   // Strip the accelerator part if any.
	   accel = strchr(originalText, '\t');
	   if (accel) *accel = 0;
	}

	USES_CONVERSION;
	IsTranslated = theApp.lang.Translate(A2CT(originalText), csTrans);
	
	if (accel) *accel = '\t';

	if (translatedText == NULL)
	{
		if (!IsTranslated) return 0;
		
		int len = csTrans.GetLength();
		if (forMenu)
		{
			accel = strchr(originalText, '\t');
			if (accel) len = len + strlen(accel);
		}
		
		return len;
	}

	if (IsTranslated)
	{
		if (accel)
		    csTrans += accel; 
		translatedText[0] = 0;
		_tcsncat(translatedText, csTrans, bufferlen);
		return csTrans.GetLength();
	}
	
	return 0;
}

BOOL GetMozillaWebBrowser(HWND hWnd, nsIWebBrowser** webBrowser)
{
	CBrowserFrame *browserFrm = (CBrowserFrame *)CWnd::FromHandle(hWnd);
	if (!browserFrm || !browserFrm->m_wndBrowserView.mWebBrowser) return FALSE;

	*webBrowser = browserFrm->m_wndBrowserView.mWebBrowser;
	NS_ADDREF(*webBrowser);
	return TRUE;
}

void AddStatusBarIcon(HWND hWnd, int id, HICON hIcon, char* tpText)
{
	CBrowserFrame *browserFrm = (CBrowserFrame *)CWnd::FromHandle(hWnd);
	if (!browserFrm) return;

	USES_CONVERSION;
	browserFrm->m_wndStatusBar.AddIcon(id);
	browserFrm->m_wndStatusBar.SetIconInfo(id, hIcon, A2CT(tpText));
}

void RemoveStatusBarIcon(HWND hWnd, int id)
{
	CBrowserFrame *browserFrm = (CBrowserFrame *)CWnd::FromHandle(hWnd);
	if (!browserFrm) return;
	
	browserFrm->m_wndStatusBar.RemoveIcon(id);
}	

BOOL InjectJS(const char* js, bool bTopWindow, HWND hWnd)
{
	CBrowserFrame *browserFrm;
	if (hWnd) 
		browserFrm = (CBrowserFrame *)CWnd::FromHandle(hWnd);
	else
		browserFrm = theApp.m_pMostRecentBrowserFrame;

	if (!browserFrm) return FALSE;
	
	nsEmbedString js2;
	NS_CStringToUTF16(nsDependentCString(js), NS_CSTRING_ENCODING_UTF8, js2);
	return browserFrm->m_wndBrowserView.InjectJS(js2.get(), bTopWindow);
}

BOOL InjectCSS(const char* css, bool bAll, HWND hWnd)
{
	CBrowserFrame *browserFrm;
	if (hWnd) 
		browserFrm = (CBrowserFrame *)CWnd::FromHandle(hWnd);
	else
		browserFrm = theApp.m_pMostRecentBrowserFrame;

	if (!browserFrm) return FALSE;

	nsEmbedString css2;
	NS_CStringToUTF16(nsDependentCString(css), NS_CSTRING_ENCODING_UTF8, css2);
	return browserFrm->m_wndBrowserView.InjectCSS(css2.get());
}

int GetKmeleonVersion()
{
	return KMELEON_VERSION;
}

kmeleonFunctions kmelFuncs = {
   SendMessage,
   GetCommandIDs,
   _NavigateTo,
   GetDocInfo,
   _GetPreference,
   SetPreference,
   SetStatusBarText,
   GetMozillaSessionHistory,
   GotoHistoryIndex,
   RegisterBand,
   CreateToolbar,
   GetID,
   GetInfoAtPoint,
   CommandAtPoint,
   GetGlobalVar,
   EncodeUTF8,
   DecodeUTF8,
   GetBrowserviewRect,
   GetMenu,
   SetForceCharset,
   SetCheck,
   Load,
   ClearCache,
   BroadcastMessage,
   ParseAccel,
   DelPreference,
   GetPreference,
   RegisterSideBar,
   ToggleSideBar,
   TranslateEx,
   GetIconList,
   GetMozillaWebBrowser,
   AddStatusBarIcon,
   RemoveStatusBarIcon,
   InjectJS,
   InjectCSS,
   GetInfoAtClick,
   GetKmeleonVersion,
   NULL,
   NavigateTo,
   Translate,
   SetGlobalVar
};

BOOL CPlugins::TestLoad(LPCTSTR file, const char *description)
{
   char preference[128] = "kmeleon.plugins.";

   USES_CONVERSION;
   strcat(preference, T2CA(file));
   strcat(preference, ".load");
   
   int load = theApp.preferences.GetBool(preference, -1);
   if (load == -1) {
      CString message, title;
	   title.LoadString(IDS_NEW_PLUGIN_FOUND_TITLE);
      message.Format(IDS_NEW_PLUGIN_FOUND, A2CT(description));

      if (MessageBox(NULL, message, _T("Plugin found"), MB_YESNO) == IDYES)
         load = 1;
      else
         load = 0;

      theApp.preferences.SetBool(preference, load);
   }
   return load;
}

BOOL CPlugins::IsLoaded(LPCTSTR pluginName)
{
   kmeleonPlugin* kPlugin;
   if (!pluginList.Lookup(pluginName, kPlugin))
      return FALSE;

   return kPlugin->loaded;
}

kmeleonPlugin * CPlugins::Load(CString file)
{
   file.TrimLeft();
   file.TrimRight();

   int filePos = file.ReverseFind(_T('\\'));
   int filePos2 = file.ReverseFind(_T('/'));
   if (filePos2>filePos) filePos = filePos2;
   ++filePos;

   CString noPath = file.Mid(filePos);
   noPath.MakeLower();

   CString pluginName;
   int extPos = noPath.ReverseFind(_T('.'));
   if (noPath.Mid(extPos).Compare(_T(".dll"))!=0)
      extPos = -1;

   if (extPos!=-1)
      pluginName = noPath.Mid(0, extPos);
   else
      pluginName = noPath;

   // check if the plugin is already loaded
   kmeleonPlugin * kPlugin;
   if (pluginList.Lookup(pluginName, kPlugin))
      return kPlugin; // it's already loaded

   HINSTANCE plugin;

   // if pattern does not contain ':' we need to prepend pluginsDir
   if (file.Find(':') == -1)
      file = theApp.preferences.pluginsDir + file;

   // we need to append .dll because NT4 gets confused if a directory in the path
   // contains a '.' and the file to be loaded does not
   if (extPos == -1)
      file +=_T(".dll");

   plugin = LoadLibrary(file);
   if (!plugin) return NULL;

   KmeleonPluginGetter kpg = (KmeleonPluginGetter)GetProcAddress(plugin, "GetKmeleonPlugin");
   if (!kpg) {
      FreeLibrary(plugin);
      return NULL;
   }

   kPlugin = kpg();

   USES_CONVERSION;
   kPlugin->hParentInstance = AfxGetInstanceHandle();
   kPlugin->hDllInstance = plugin;
   kPlugin->kFuncs = &kmelFuncs;
   kPlugin->dllname = safe_strdup(T2CA(pluginName));

   int loaded = kPlugin->loaded = TestLoad(pluginName, kPlugin->description);

   if (kPlugin->version < KMEL_PLUGIN_VER_MAJOR) {
      CString error;
      error.Format(IDS_OLD_PLUGIN, kPlugin->description);
      AfxMessageBox(error);
      loaded = false;
   }

   // If the plugin is enabled, tell it to Init
   if (loaded) {
      if (kPlugin->DoMessage(kPlugin->dllname, "* Plugin Manager", "Load", 0, 0) == -1)
		  loaded = false;
   }

   if (!loaded) {
   // otherwise, make a copy of the descripion, and unload it
   //else {
      kmeleonPlugin *temp = new kmeleonPlugin;

      char *sBuf = new char[strlen(kPlugin->description)+1];
      temp->description = sBuf;
      strcpy(temp->description, kPlugin->description);

      temp->dllname = kPlugin->dllname;
      temp->loaded = false;

      kPlugin=temp;

      FreeLibrary(plugin);
   }

   pluginList.SetAt(pluginName, kPlugin);
   return kPlugin;
}

int CPlugins::FindAndLoad(const TCHAR *pattern)
{
   CString filepath;
   CFileFind finder;
   BOOL bWorking;
   
   int x=_tcslen(pattern);
   while (x>0 && (pattern[x] != ':')) x--;
   
   if (x==0) {       // if pattern does not contain ':' we need to prepend pluginsDir
      CString search = theApp.preferences.pluginsDir + pattern;
      bWorking = finder.FindFile(search);
   }
   else bWorking = finder.FindFile(pattern);

   int i = 0;
   while (bWorking) {
      bWorking = finder.FindNextFile();

      filepath = finder.GetFilePath();
      if ( Load(filepath) )
         i++;
   }
   //   SendMessage("*", "* Plugin Manager", "Init");
   return i;
}

void CPlugins::UnLoadAll()
{
   SendMessage("*", "* Plugin Manager", "Exit");

   POSITION pos = pluginList.GetStartPosition();
   kmeleonPlugin * kPlugin;
   CString s;
   while (pos) {
      pluginList.GetNextAssoc(pos, s, kPlugin);
      if (kPlugin) {
         delete kPlugin->dllname;
         if (kPlugin->loaded) {
            FreeLibrary(kPlugin->hDllInstance);
         }
         else  { // the plugin was disabled, delete the copied description
            delete kPlugin->description;
            delete kPlugin;
         }
      }
   }
   pluginList.RemoveAll();
   currentCmdID = PLUGIN_COMMAND_START_ID;
}

int CPlugins::GetConfigFiles(configFileType *configFiles, int maxFiles)
{
   int numFiles = 0;
   POSITION pos = pluginList.GetStartPosition();
   kmeleonPlugin * kPlugin;
   CString s;
   while (pos) {
      pluginList.GetNextAssoc( pos, s, kPlugin);

      if (kPlugin->loaded && kPlugin->DoMessage) {
         configFileType *tempConfigFiles;
         int numTempConfigFiles=0;
         kPlugin->DoMessage(kPlugin->dllname, "* Plugin Manager", "GetConfigFiles", (long)&tempConfigFiles, (long)&numTempConfigFiles);
         int i = 0;
         while (numFiles < maxFiles && i < numTempConfigFiles) {
            memcpy(&configFiles[numFiles++], &tempConfigFiles[i++], sizeof(configFileType));
         }
      }
   }
   return numFiles;
}
